<?php

namespace Drupal\frontend_routes\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\Routing\Exception\RouteNotFoundException;

/**
 * Configure Frontend routes settings for this site.
 */
class RoutesSettingsForm extends ConfigFormBase
{

    const SETTINGS = 'frontend_routes.settings';

    /**
     * {@inheritdoc}
     */
    public function getFormId()
    {
        return 'frontend_routes_settings';
    }

    /**
     * {@inheritdoc}
     */
    protected function getEditableConfigNames()
    {
        return [static::SETTINGS];
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(array $form, FormStateInterface $form_state)
    {

        $config = $this->config(static::SETTINGS);

        $config_routes_list = $config->get('routes_list');
        $form['routes_list'] = [
            '#type' => 'textarea',
            '#title' => $this->t('List Routes'),
            '#description' => 'Add to list only used routes! For example: user.login ... etc.',
            '#default_value' => $config_routes_list,

        ];

        // Check if the input is empty
        if (strlen(trim($config_routes_list)) != 0) {

            $routes = $this->convertToArray($config_routes_list);

            if (is_array($routes)) {

                $config_routes = [];
                if (count($config->get('routes')) !== 0)
                    $config_routes = $this->modifyRoutesToBuildForm($config->get('routes'));

                $form['routes'] = [
                    '#type' => 'container',
                    '#tree' => TRUE,
                ];

                $route_provider = \Drupal::service('router.route_provider');

                foreach ($routes as $key => $route_name) {

                    try {
                        $route = $route_provider->getRouteByName($route_name);

                        if ($route !== NULL) {

                            $form['routes'][$route_name] = array(
                                '#type' => 'fieldset',
                                '#title' => $route_name,
                            );

                            $form['routes'][$route_name]['checkbox'] = array(
                                '#type' => 'checkbox',
                                '#title' => $this->t('Path is active') . ': ' . $route->getPath(),
                                '#default_value' => $config_routes[$route_name]['checkbox'] ?? false,
                            );

                            $form['routes'][$route_name]['path'] = array(
                                '#type' => 'textfield',
                                '#default_value' => $config_routes[$route_name]['path'] ?? $route->getPath(),        // custom path
                                '#states' => [
                                    'visible' => [
                                        [':input[name="checkbox"]' => ['value' => true]],
                                    ],
                                ],
                            );

                        }
                    } catch (RouteNotFoundException $exception) {
//                      return NULL;
                    }
                }
            }
        }

        return parent::buildForm($form, $form_state);
    }

    /**
     * Convert Text to Array
     * With form TextArray
     */
    private function convertToArray($text)
    {

        $text = preg_replace('/\s+/', ' ', trim($text));

        $array = array_map('trim', explode(' ', $text));

        return $array;
    }

    /**
     * {@inheritDoc}
     */
    private function modifyRoutesToBuildForm($value)
    {
        $routes = [];

        foreach ($value['routes'] as $key => $route) {
            $routes[$route['name']] = [
                'checkbox' => $route['checkbox'],
                'path' => $route['path']
            ];
        }

        return $routes;
    }

    /**
     * {@inheritdoc}
     */
    public function validateForm(array &$form, FormStateInterface $form_state)
    {
        parent::validateForm($form, $form_state);
    }

    /**
     * {@inheritdoc}
     */
    public function submitForm(array &$form, FormStateInterface $form_state)
    {

        //modify to config schema
        $routes = $this->modifyRoutesToConfig($form_state->getValue('routes'));

        $this->config(static::SETTINGS)
            ->set('routes_list', $form_state->getValue('routes_list'))
            ->set('routes', $routes)
            ->save();

        // Clear all plugin caches.
        \Drupal::service('plugin.cache_clearer')->clearCachedDefinitions();

        // Rebuild routers based on all rebuilt data.
        \Drupal::service('router.builder')->rebuild();

        parent::submitForm($form, $form_state);
    }

    /**
     * Convert array
     */
    private function modifyRoutesToConfig($value)
    {
        $routes = [];

        foreach ($value as $route_name => $route) {
            $routes['routes'][] = [
                'name' => $route_name,
                'checkbox' => $route['checkbox'],
                'path' => $route['path']
            ];
        }

        return $routes;
    }

}